
# q-expansion of newform 75.12.b.d, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 75
weight = 12
poly_data = [202500, 0, 901, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 451, 0, 1], 450], [[451, 0, 1, 0], 1]]

hecke_ring_character_values = [[26, [1, 0, 0, 0]], [52, [-1, 0, 0, 0]]]
aps_data = [[0, 1, -6, 0], [0, 0, 243, 0], [0, 0, 0, 0], [0, 3584, 5684, 0], [132296, 0, 0, 30976], [0, 71936, -292778, 0], [0, -78080, 4250934, 0], [-8748196, 0, 0, -131584], [0, -1571328, 14134872, 0], [98829998, 0, 0, 4221952], [-38748600, 0, 0, 6440192], [0, -12830976, 346513754, 0], [-173020790, 0, 0, 18386432], [0, -3544576, -1598048348, 0], [0, -7918592, 1022573064, 0], [0, 111944192, 1208121822, 0], [859489096, 0, 0, -240207616], [-4190848802, 0, 0, 116806144], [0, -213015040, 11999581244, 0], [-9697752608, 0, 0, -823383040], [0, -909419008, -13394501138, 0], [-11482822200, 0, 0, 640648960], [0, 562434048, -23726037468, 0], [-40433437506, 0, 0, 1657191936], [0, -89902080, -18582564766, 0], [-103273185894, 0, 0, -2425227264], [0, 1226179584, 25627143772, 0], [0, 4144946688, -75213083196, 0], [-20798363486, 0, 0, 2396491776], [0, 5822605568, -81004512318, 0], [0, -2155258880, 71001048764, 0], [128199442488, 0, 0, -16028142336], [0, 621947136, -566926701306, 0], [-54116143604, 0, 0, 30924670464], [384195339422, 0, 0, 13101242368], [800695667992, 0, 0, 26784628480], [0, -4956466944, -690967420246, 0], [0, 73646254080, -1058484628748, 0], [0, 107344779264, 650733906264, 0], [0, -85793943552, 246721469622, 0], [889445984320, 0, 0, 47143865600], [497003847590, 0, 0, -101532423168], [373159660216, 0, 0, 128322192896], [0, 8090208768, -2940584298218, 0], [0, 4336922112, 3775180927554, 0], [1624677107192, 0, 0, -27914874112], [-4558784855380, 0, 0, 304592174592], [0, -2481398784, -9948448750028, 0], [0, -287489399296, -814714038636, 0], [-6088974006894, 0, 0, -188971245056], [0, -625791580416, 6221377734642, 0], [8087629585384, 0, 0, 440279998976], [243474485986, 0, 0, -645039970304], [3086683143312, 0, 0, 1249525006080], [0, 17716547328, 6515559407454, 0], [0, 10755374080, 32771993712792, 0], [-4257437533586, 0, 0, 1565685518336], [-11424620543984, 0, 0, -1455694648064], [0, 1059777274112, -7286979747046, 0], [45682482052194, 0, 0, -177018419712], [0, -680411400704, -9326701189988, 0], [0, 1317052147200, -19975544229858, 0], [0, 309192506368, -60130094633836, 0], [63610419470976, 0, 0, -436015593984], [0, 1580910328320, -28457819502098, 0], [0, 325666247168, -34940277933846, 0], [21172670505612, 0, 0, -5558222248960], [0, -4475388174848, 2208521060954, 0], [0, -3377556861440, 6290673496764, 0], [-22658289943102, 0, 0, -2219313822208], [0, -3258759508224, 1229222684202, 0], [-38465264361624, 0, 0, -1360852179456], [0, 2748812989952, 162818006490284, 0], [0, 7007920327424, 22033814628142, 0], [109109460582380, 0, 0, 2240830561280], [0, 1210809008640, -80012321872728, 0], [151716828631806, 0, 0, 3739702401024], [0, 2378494720768, -96893648743246, 0], [-193147045296582, 0, 0, -4877124811776], [141482685302326, 0, 0, 5267099787264], [-25515931030696, 0, 0, -9038565734144], [-10785497127026, 0, 0, 5003910005248], [106110995293200, 0, 0, 244194104832], [0, 10930786956800, 115568796157702, 0], [-205283006512360, 0, 0, 3883048313600], [0, 14369483521536, -170406214551948, 0], [-272569005408698, 0, 0, 20001097161728], [0, -3167440756736, -283538248786966, 0], [-384370969480518, 0, 0, -3004195768320], [0, 1084233301504, -64574103448628, 0], [0, -10977895724032, 148316294649564, 0], [770811558331800, 0, 0, 8817399828480], [0, 1884583772160, 418070137545884, 0], [195869322070528, 0, 0, 11245990340864], [-449796404664196, 0, 0, -8577168845824], [0, 30959343379456, -636633192999168, 0], [-446236326055634, 0, 0, 46966294744064], [-12548327893790, 0, 0, 24276006413312], [0, -1095727277056, -1178754598516628, 0], [-1480641484526370, 0, 0, -18897344263168], [0, -18827160766976, 527482230194084, 0], [0, -19747160878080, -777081640033206, 0], [0, 81555387452928, 892324071536412, 0], [40797753156894, 0, 0, -5039034537984], [-1748235563311204, 0, 0, 12110522305536], [0, -817549962240, -94742622664126, 0], [0, 3445157242368, 960898401090084, 0], [0, 10049444349184, -714161037324078, 0], [-504173036128408, 0, 0, 31797456905728], [-1658382927460774, 0, 0, 46934026070016], [0, 68756564569600, 1649829411696644, 0], [0, -72783313629952, 49772175016102, 0], [0, -24275856178944, 3023354078225214, 0], [-938661811384004, 0, 0, -83912906374656], [-114474422329808, 0, 0, -21854117262080], [2203738139781570, 0, 0, -27895474856448], [0, -137222083337216, -1601330139309068, 0], [0, 86309140096000, -2642494267127376, 0], [0, -42698649082880, 3748954404734742, 0], [-1082026264605080, 0, 0, -170325712764160], [-785827862170778, 0, 0, 10753353879040], [0, 152053052396032, 161351461558822, 0], [0, 199561500828672, 2557072543095714, 0], [0, 58127012951040, 6003758207582892, 0], [5055287352572356, 0, 0, 126127817575936], [5279169962733298, 0, 0, 66012198800384], [5854179261763874, 0, 0, -80667996527104], [-335284176198048, 0, 0, 168214378449408], [0, 247319090820096, -1781498701209316, 0], [0, 221068686852864, 1569319427306062, 0], [5632660345840556, 0, 0, -67942535981056], [0, 157201009908224, 1646573417691312, 0], [-4546464029880584, 0, 0, 116978526167296], [0, 328196142413056, -3949767043757926, 0], [4759993124747394, 0, 0, 179548466439168], [6252395266144702, 0, 0, 57317122556928], [0, -268624629404160, 3486986121088782, 0], [0, -571656644298752, -3997863177397156, 0], [0, -420076875830272, 4118927022714714, 0], [2236601363467910, 0, 0, -23411320286720], [6729014238869412, 0, 0, -290268833136640], [-8062847203206246, 0, 0, -533579156462592], [0, 296962870499328, -6748502516199548, 0], [0, -141829792870400, -11158964565212196, 0], [8575548964461330, 0, 0, -32399158077440], [-2628966929387560, 0, 0, -160789391352320], [0, 35317897150720, -3562079677630538, 0], [0, -585008277677312, 11637649111657134, 0], [-13560263290460020, 0, 0, 53945306240000], [0, 1165326813558784, -1498696934818608, 0], [0, -523282751362816, 1945727956950794, 0], [-18620643457410646, 0, 0, 16053743676928], [0, -279426258564096, -2968120764426188, 0], [0, 712293746654208, -6236487483194856, 0], [0, 664737188069888, 7282288134731804, 0], [26094400931169632, 0, 0, 200447765862400], [4075073996941040, 0, 0, 18503266430720], [-27483936827270266, 0, 0, 53085624395776], [0, -174644576386048, 15444408123201674, 0], [-10126057076278550, 0, 0, 786685216704512], [0, -343118990905344, -26740995198155196, 0], [0, -712419144358144, -30245472725983398, 0], [0, -206016466176000, -10567422521570476, 0], [20016898462373760, 0, 0, -394835450750208], [0, -187279581221120, -22563417768659706, 0], [0, -1379118935152128, -7791513837202128, 0], [28883227376211632, 0, 0, 1014015752405760], [0, -863906066124544, -24929750013373966, 0]]
